function [u,A,b]=WaermeGlLoes (T,c,u0,ul,ur,m,n)
% Loesung der 1D-Waermeleitungsgleichung: u_t - c*u_xx = 0
% auf dem Gebiet (x,t) in [0,1] \times [0,T]. Ort und Zeit werden 
% diskretisiert und das komplette Gleichungssystem geloest.
% Dabei wird das Zeitintervall [0,T] mit linspace(T,0,m)
%        und das Ortsintervall [0,1] mit linspace(0,1,n) diskretisiert
% Input:
%      T     Endzeitpunkt
%      c     Konstante c in Gleichung
%     u0     Funktion u0(x) fuer Anfangswerte u(x,0)
%     ul     Funktion ul(t) fuer linke Randwerte u(0,t)
%     ur     Funktion ur(t) fuer rechte Randwerte u(1,t)
%      m     linspace(T,0,m) fuer t-Diskretisierung
%      n     linspace(0,1,n) fuer x-Diskretisierung
% Output:
%      u     (m,n) Matrix mit u Werten
%            u(1,:) sind die Werte bei x=linspace(0,1,n) und t=T
%      A     sparse Matrix fuer LGS
%      b     rechte Seite fuer LGS
u=NaN*ones(m,n);
uSize=size(u);
h=1/(n-1); tau=T/(m-1); 
N=m*n;
A=spalloc(N,N,4*N); b=zeros(N,1);
glNo=0;
stencil=[2*c/(h^2)+1/tau, -1*c/(h^2), -1*c/(h^2) , -1/tau];
tdisk=linspace(T,0,m);
xdisk=linspace(0,1,n);
% Innere
for z=1:m-1
    for s=2:n-1
        n_self=sub2ind(uSize,z,s);
        n_right=n_self+m;n_left=n_self-m;
        n_down=n_self+1;
        glNo=glNo+1;
        A(glNo,[n_self,n_left,n_right,n_down]) = stencil;
    end
end
% linker Rand
value=ul(tdisk);
for z=1:m-1
    n_self=sub2ind(uSize,z,1);
    glNo=glNo+1;
    A(glNo,n_self)=1; b(glNo)=value(z);
end
% rechter Rand
value=ur(tdisk);
for z=1:m-1
    n_self=sub2ind(uSize,z,n);
    glNo=glNo+1;
    A(glNo,n_self)=1; b(glNo)=value(z);
end
% unterer Rand
value=u0(xdisk);
for s=1:n
    n_self=sub2ind(uSize,m,s);
    glNo=glNo+1;
    A(glNo,n_self)=1; b(glNo)=value(s);
end
assert(glNo==N); assert(size(b,1)==N); 
assert(size(A,1)==N); assert(size(A,2)==N);
u=reshape(A\b,size(u));
end